package db

import (
	"context"
	"errors"
	"fmt"
	"os"
	"strings"

	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type MongoDB struct {
	Client   *mongo.Client
	Database *mongo.Database

	Databases map[string]*mongo.Database //所有数据库
}

var MongoClient *MongoDB

func (db *MongoDB) GetCollection(name string) *mongo.Collection {
	return db.Database.Collection(name)
}

func (db *MongoDB) GetDbCollection(dbname string, name string) *mongo.Collection {
	if len(dbname) < 1 {
		return db.GetCollection(name)
	}
	return db.GetOrCreateDatabase(dbname).Collection(name)
}

func (db *MongoDB) GetOrCreateDatabase(name string) *mongo.Database {
	datab := db.Databases[name]
	if datab == nil {
		datab = db.Client.Database(name)
		db.Databases[name] = datab
	}
	return datab
}

func NewMongoDB() *MongoDB {

	inst, err := NewMongoDBFromConfig("nrhe-mongo")

	if err != nil {
		panic(err)
	}

	MongoClient = &MongoDB{Client: inst.Client, Database: inst.Database, Databases: map[string]*mongo.Database{}}
	return MongoClient
}

func createMongo(mongodns string) (*MongoDB, error) {
	start := strings.LastIndex(mongodns, "/")
	end := strings.LastIndex(mongodns, "?")
	database := mongodns[start+1 : end]
	fmt.Println("mongo-format database => ", database)
	if len(database) < 1 {
		return nil, errors.New("config is not format of mongodb://root:root@172.17.0.1:37017/comm?authSource=admin")
	}
	clientOptions := options.Client().ApplyURI(mongodns)
	// 连接到MongoDB
	mgoCli, err := mongo.Connect(context.TODO(), clientOptions)
	if err != nil {
		panic(err)
	}

	// 检查连接
	err = mgoCli.Ping(context.TODO(), nil)
	if err != nil {
		panic(err)
	}
	return &MongoDB{Client: mgoCli, Database: mgoCli.Database(database)}, nil
}

func NewMongoDBFromConfig(mongoConfKey string, timeout ...interface{}) (*MongoDB, error) {
	//mongodb://root:root@172.17.0.1:37017/comm?authSource=admin
	// mongodns, err := bus.RequestConfig(mongoConfKey)
	// if err != nil {
	// 	return nil, err
	// }
	mongodns := "mongodb://root:nrhe_zzzxxx@127.0.0.1:17017/nrhe?authSource=admin"

	mongoenv := os.Getenv("MONGO")
	if len(mongoenv) > 0 {
		mongodns = mongoenv
	}
	fmt.Println("config mongodns => ", mongodns)
	fmt.Println("env mongodns => ", mongodns)

	return createMongo(mongodns)
}
